package com.gic.haoban.manage.web.controller.licence;


import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderPageDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderQDTO;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.licence.LicenceOrderApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.licence.LicenceOrderPageQO;
import com.gic.haoban.manage.web.qo.licence.LicenceOrderQO;
import com.gic.haoban.manage.web.vo.licence.LicenceOrderPageVO;
import com.gic.haoban.manage.web.vo.licence.LicenceOrderVO;
import com.gic.marketing.api.dto.AccountChargeInfoDTO;
import com.gic.marketing.api.enums.AccountChargeSceneEnum;
import com.gic.marketing.api.service.EnterpriseAccountInfoApiService;
import com.gic.thirdparty.api.dto.ScanPayReqDataDTO;
import com.gic.thirdparty.api.service.Pay4WXService;
import com.gic.thirdparty.api.service.RechargeCenterService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 企业微信许可账号购买
 * @author jx
 * @since 2023-03-09
 */
@RestController
@RequestMapping("/licence-order")
public class LicenceOrderController {

    private static Logger logger = LogManager.getLogger(LicenceOrderController.class);

    @Autowired
    private LicenceOrderApiService licenceOrderApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private EnterpriseAccountInfoApiService enterpriseAccountInfoApiService;
    @Autowired
    private Pay4WXService pay4WXService;
    @Autowired
    private RechargeCenterService rechargeCenterService;
    /**
     * 查询订单详情
     */
    @RequestMapping("licence-order-detail")
    public RestResponse<LicenceOrderVO> getLicenceOrderDetail(@RequestParam Long orderId) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        ServiceResponse<LicenceOrderDTO> licenceOrderDetail = licenceOrderApiService.getLicenceOrderDetail(orderId,1);
        LicenceOrderVO licenceOrderVO = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderDetail.getResult()), LicenceOrderVO.class);
        return RestResponse.successResult(licenceOrderVO);
    }
    /**
     * 删除订单
     */
    @RequestMapping("licence-order-delete")
    public RestResponse<Boolean> deleteLicenceOrder(@RequestParam Long orderId) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        ServiceResponse<Boolean> booleanServiceResponse = licenceOrderApiService.deleteLicenceOrder(orderId);
        return RestResponse.successResult(booleanServiceResponse.getResult());
    }
    /**
     * 新增或者修改订单
     */
    @RequestMapping("licence-order-saveOrUpdate")
    public RestResponse<String> saveOrUpdateLicenceOrder(@RequestBody LicenceOrderQO licenceOrderQO) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        //金额校验
        Integer integer = licenceOrderQO.checkPrice();
        if (integer==-1){
            return RestResponse.failure("-9999","许可账号数量有误！");
        } else if (integer==0) {
            return RestResponse.failure("-9999","购买金额计算有误！");
        }
        LicenceOrderQDTO licenceOrderQDTO = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderQO), LicenceOrderQDTO.class);
        licenceOrderQDTO.setEnterpriseId(loginUser.getEnterpriseId());
        licenceOrderQDTO.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        licenceOrderQDTO.setCreatorId(loginUser.getClerkId());
        licenceOrderQDTO.setCreatorName(loginUser.getClerkName());
        String result = licenceOrderApiService.saveOrUpdateLicenceOrder(licenceOrderQDTO).getResult();
        return RestResponse.successResult(result);
    }
    /**
     * 分页查询订单列表
     */
    @RequestMapping("licence-order-page")
    public RestResponse<Page<LicenceOrderPageVO>> getLicenceOrderPage(@RequestBody LicenceOrderPageQO licenceOrderPageQO) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        String enterpriseId = loginUser.getEnterpriseId();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String clerkName = loginUser.getClerkName();
        LicenceOrderPageQDTO qdto = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderPageQO), LicenceOrderPageQDTO.class);
        qdto.setEnterpriseId(enterpriseId);
        qdto.setWxEnterpriseId(wxEnterpriseId);
        qdto.setCreatorName(clerkName);
        ServiceResponse<Page<LicenceOrderPageDTO>> licenceOrderPage = licenceOrderApiService.getLicenceOrderPage(qdto);
        Page<LicenceOrderPageVO> result = PageHelperUtils.changePageToCurrentPage(licenceOrderPage.getResult(), LicenceOrderPageVO.class);
        return RestResponse.successResult(result);
    }
    /**
     * 取消订单/订单已经过期
     */
    @RequestMapping("licence-order-cancel")
    public RestResponse<Boolean> cancelLicenceOrder(@RequestParam Long orderId,Integer type) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        String enterpriseId = loginUser.getEnterpriseId();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String clerkName = loginUser.getClerkName();
        Boolean result = licenceOrderApiService.updateLicenceOrderType(orderId, type,enterpriseId,wxEnterpriseId,clerkName).getResult();
        return RestResponse.successResult(result);
    }
    /**
     * 订单凭证上传
     */
    @RequestMapping("licence-order-voucher-upload")
    public RestResponse<Boolean> uploadLicenceOrderVoucher(@RequestParam Long orderId,String voucher) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        String enterpriseId = loginUser.getEnterpriseId();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String clerkName = loginUser.getClerkName();
        Boolean result = licenceOrderApiService.uploadLicenceOrderVoucher(orderId, voucher,enterpriseId,wxEnterpriseId,clerkName).getResult();
        return RestResponse.successResult(result);
    }
    /**
     * 支付订单
     */
    @RequestMapping("licence-order-pay-1")
    public RestResponse<Map<String, Object>> payLicenceOrder(@RequestParam Long orderId,@RequestParam(defaultValue = "0") int accountType) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        LicenceOrderDTO result = licenceOrderApiService.getLicenceOrderDetail(orderId, 1).getResult();
        if (result == null){
            return  RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_5.getCode()),HaoBanErrCode.ERR_5.getMsg());
        }
        Integer money = result.getPrice();
        // 交易订单号
        String transactionCode = ToolUtil.randomStringByTime();
        ScanPayReqDataDTO scanPayReqData = new ScanPayReqDataDTO("好办平台企业微信许可账号购买", loginUser.getEnterpriseId(),
                transactionCode, money, DateUtil.dateToStr(new Date(), DateUtil.FORMAT_DATETIME_14), "",
                "NATIVE");
        String accountDepartId = "";
        if(accountType==0) {
            AccountChargeInfoDTO dto =  this.enterpriseAccountInfoApiService.getAccountDeparment(loginUser.getEnterpriseId(), AccountChargeSceneEnum.creator_only, null , loginUser.getClerkId()) ;
            logger.info("充值账号信息={}",dto);
            accountDepartId = dto.getAccountDepartId() ;
        }
        if(accountType==1) {
            accountDepartId = "1" ;
        }
        scanPayReqData.setAccountType(accountType);
        scanPayReqData.setAccountDepartId(accountDepartId);
        //二维码
        String url = pay4WXService.createPrePaymentOrderForWX(scanPayReqData);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("transactionCode", transactionCode);
        map.put("url", url);
        map.put("money",money);
        // TODO: 2023/3/21 记录订单号transactionCode
        return RestResponse.successResult(map);
    }
    /**
     * 支付订单-判断是否支付成功
     */
    @RequestMapping("licence-order-pay-2")
    public RestResponse<Boolean> payLicenceOrder2(@RequestParam String transactionCode) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        if (StrUtil.isBlank(transactionCode)){
            return  RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_5.getCode()),HaoBanErrCode.ERR_5.getMsg());
        }
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("enterpriseId", loginUser.getEnterpriseId());
        params.put("out_trade_no", transactionCode);
        boolean flag = rechargeCenterService.isRechargeSuccess(params);
        logger.info("定时刷新(判断是否充值成功) FLAG====> [{}]", flag);
        return RestResponse.successResult(flag);
    }

    /**
     * 查询企业品牌名称
     */
    @RequestMapping("get-enterprise-name")
    public RestResponse<HashMap<String, String>> getEnterpriseName() {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        HashMap<String, String> result = new HashMap<>(8);
        //查询gic品牌名称
        EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(loginUser.getEnterpriseId());
        if (ObjectUtil.isNotNull(enterpriseDTO)){
            logger.info("gic企业查询返回实体：{}", JSON.toJSONString(enterpriseDTO));
            result.put("enterpriseName",enterpriseDTO.getEnterpriseName());
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(loginUser.getWxEnterpriseId());
        if (ObjectUtil.isNotNull(wxEnterpriseDTO)){
            logger.info("企业微信查询返回实体：{}", JSON.toJSONString(wxEnterpriseDTO));
            result.put("wxEnterpriseName",wxEnterpriseDTO.getCorpName());
        }
        return RestResponse.successResult(result);
    }
    /**
     * 查询企业是否存在未支付订单
     */
    @RequestMapping("licence-order-isPay")
    public RestResponse<Boolean> isPayLicenceOrder() {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        if (loginUser == null || StringUtils.isBlank(loginUser.getEnterpriseId())){
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_4.getCode()),HaoBanErrCode.ERR_4.getMsg());
        }
        Boolean result = licenceOrderApiService.isPayLicenceOrder(loginUser.getWxEnterpriseId()).getResult();
        return RestResponse.successResult(result);
    }


}
